package cn.com.ty.lift.business.framework.aspect;

import cn.com.ty.lift.common.verify.Ver;
import cn.com.ty.lift.common.verify.Verifier;
import cn.com.ty.lift.common.verify.VerifyProcessor;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;

import java.lang.reflect.Method;
import java.lang.reflect.Parameter;

/**
 * the {@link MethodInterceptor} for verify parameter.
 *
 * @author wcz
 * @since 2020/3/15
 */
public class VerifyMethodInterceptor implements MethodInterceptor {
    @Override
    public Object invoke(MethodInvocation invocation) throws Throwable {
        //check whether the method is present with verifier annotation.
        Method method = invocation.getMethod();
        boolean annotationPresent = method.isAnnotationPresent(Verifier.class);
        if(!annotationPresent){
            return invocation.proceed();
        }
        //check the list of parameters.
        Parameter[] parameters = method.getParameters();
        if(null == parameters || parameters.length == 0){
            return invocation.proceed();
        }
        //if the parameter is present with Ver. return the index.
        Object object = null;
        Object[] arguments = invocation.getArguments();
        for (int i = 0; i < parameters.length; i++) {
            if(parameters[i].isAnnotationPresent(Ver.class)){
                object = arguments[i];
                break;
            }
        }
        if(null == object){
            return invocation.proceed();
        }
        Verifier verifier = method.getDeclaredAnnotation(Verifier.class);
        VerifyProcessor.perform(object, verifier.fields());
        return invocation.proceed();
    }
}
