package cn.com.ty.lift.business.maintenance.controller;

import cn.com.ty.lift.business.evaluation.dao.entity.Evaluation;
import cn.com.ty.lift.business.evaluation.service.EvaluationService;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceOption;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceRecord;
import cn.com.ty.lift.business.maintenance.dao.entity.MtRecordCost;
import cn.com.ty.lift.business.maintenance.dao.entity.MtRecordImg;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtPlanRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtOptionTree;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtPlanResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtRecordResponse;
import cn.com.ty.lift.business.maintenance.service.*;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.common.model.CountPage;
import cn.com.ty.lift.common.utils.ValuePool;
import cn.com.ty.lift.common.verify.Ver;
import cn.com.ty.lift.common.verify.Verifier;
import cn.com.ty.lift.common.verify.Verify;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.base.Splitter;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 　* <p>
 * 维保管理-日常保养
 * <p>
 *
 * @author wcz
 * 　* @since 2020/1/17 17:24
 *
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("maintenance/record")
public class MaintenanceRecordController {

    private MaintenancePlanService      maintenancePlanService;
    private MaintenanceRecordService    maintenanceRecordService;
    private MtRecordCostService         mtRecordCostService;
    private MtRecordImgService          mtRecordImgService;
    private MaintenanceOptionService    maintenanceOptionService;
    private ProjectLiftRelevanceService projectLiftRelevanceService;
    private EvaluationService           evaluationService;

    //法规维保间隔15天
    private static final int maintenance_interval = 15;

    /**
     * 根据条件分页查询维保-日常保养
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("pageRecord")
    @Verifier(fields = {"mtCompanyId"})
    public RestResponse pageRecord(@Ver @RequestBody MtRecordRequest request) {
        IPage<MtRecordResponse> pages = maintenanceRecordService.pageRecordByCondition(request);
        long count = maintenanceRecordService.countRecordByCondition(request);
        return RestResponse.success(CountPage.getCountPage(count, pages));
    }

    /**
     * 根据条件分页查询超期的维保计划
     *
     * @param request MtPlanRequest
     * @return RestResponse
     */
    @PostMapping("pagePlan")
    @Verifier(fields = {"mtCompanyId"})
    public RestResponse pagePlan(@Ver @RequestBody MtPlanRequest request) {
        IPage<MtPlanResponse> pages = maintenancePlanService.pagePlanByCondition(request);
        Verify.notNull(pages);
        List<MtPlanResponse> records = pages.getRecords();
        if (IterUtil.isNotEmpty(records)) {
            List<ProjectLiftRelevance> relevances = projectLiftRelevanceService.listByMtCompany(request.getMtCompanyId());
            records.forEach(record -> {
                Optional<ProjectLiftRelevance> liftRelevance = relevances
                        .stream().filter(r -> (r.getLiftId().equals(record.getLiftId()) && r.getProjectId().equals(record.getProjectId()))).findFirst();
                //如果存在设置的计划间隔
                if (liftRelevance.isPresent()) {
                    ProjectLiftRelevance relevance = liftRelevance.get();
                    int subtract = maintenance_interval - relevance.getPlanInterval();
                    //如果设置的计划间隔 >= 法规间隔，直接判断为法规超期；超期类型，1计划超期，2法规超期
                    if (0 >= subtract) {
                        record.setOverdue(2);
                    } else {
                        log.info("PlanDate: {}", record.getPlanDate());
                        LocalDate deadline = record.getPlanDate().plusDays(subtract);
                        log.info("deadline: {}", deadline);
                        //如果超过法规15天，法规超期LocalDate.now() < deadline
                        LocalDateTime max = LocalDateTime.of(deadline, LocalTime.MAX);
                        log.info("maxtime: {}", max);
                        record.setOverdue(LocalDateTime.now().isBefore(max) ? 1 : 2);
                        log.info("overdue: {}", record.getOverdue());
                    }
                } else {
                    //fixme：如果没有设置计划间隔，暂时判定为计划超期
                    record.setOverdue(1);
                }
            });
        }
        long count = maintenancePlanService.countPlanByCondition(request);
        return RestResponse.success(CountPage.getCountPage(count, pages));
    }

    /**
     * 根据id查找,带关联信息
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("one")
    @Verifier(fields = {"id"})
    public RestResponse one(@Ver @RequestBody MtRecordRequest request) {
        MtRecordResponse entity = maintenanceRecordService.infoById(request);
        Verify.notNull(entity);
        /**
         * 维保项id:0/1/2, 0:无需保养，1：已保养，2：需要更换
         * 保养项目  用键值对的方式存储，如(1:0,2:1,3:1...)
         */
        String maintenanceOption = entity.getMaintenanceOption();
        if (StrUtil.isNotEmpty(maintenanceOption)) {
            Map<String, String> opts = Splitter.on(ValuePool.COMMA).withKeyValueSeparator(ValuePool.COLON).split(maintenanceOption);
            Map<Long, Integer> optMap = new HashMap<>();
            opts.forEach((key, value) -> {
                optMap.put(Long.valueOf(key), Integer.valueOf(value));
            });
            Collection<MaintenanceOption> mtOptions = maintenanceOptionService.listByIds(optMap.keySet());
            mtOptions.forEach(option -> {
                option.setOpt(optMap.get(option.getId()));
            });
            entity.setMtOptions(mtOptions);
        }
        Long id = request.getId();
        //查询费用项
        entity.setMtRecordCosts(mtRecordCostService.listByRecord(id));
        //总计 partsCost, manCost, 拼接sparepart
//        List<MtRecordCost> mtRecordCosts = entity.getMtRecordCosts();
//        if(IterUtil.isNotEmpty(mtRecordCosts)){
//            BigDecimal partsCost = mtRecordCosts.stream().map(MtRecordCost::getPartsCost).reduce(BigDecimal.ZERO, (m1, m2) -> m1.add(m2));
//            BigDecimal manCost = mtRecordCosts.stream().map(MtRecordCost::getManCost).reduce(BigDecimal.ZERO, (m1, m2) -> m1.add(m2));
//            String sparepart = mtRecordCosts.stream().map(MtRecordCost::getSparepart).collect(Collectors.joining(";"));
//            entity.setPartsCost(partsCost);
//            entity.setManCost(manCost);
//            entity.setSparepart(sparepart);
//        }
        //查询图片
        entity.setMtRecordImgs(mtRecordImgService.listByRecord(id));
        //评价
        entity.setEvaluation(evaluationService.findByRecord(id));
        return RestResponse.success(entity);
    }

    /**
     * 查看维保单图片list
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("images")
    @Verifier(fields = {"id"})
    public RestResponse images(@Ver @RequestBody MtRecordRequest request) {
        List<MtRecordImg> mtRecordImgs = mtRecordImgService.listByRecord(request.getId());
        return RestResponse.success(mtRecordImgs);
    }

    /**
     * 维保项 - 树形
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("treeOptions")
    @Verifier(fields = {"mtCompanyId", "type", "liftType"})
    public RestResponse treeOptions(@Ver @RequestBody MtRecordRequest request) {
        Integer type = request.getType();
        Integer liftType = request.getLiftType();
        Long mtCompanyId = request.getMtCompanyId();
        List<MaintenanceOption> optionList = maintenanceOptionService.listByTypeAndLiftType(type, liftType, mtCompanyId);
        Verify.notNull(optionList);
        //按照sort groupby
        Map<Integer, List<MaintenanceOption>> mtOptionMap = optionList.stream().collect(Collectors.groupingBy(MaintenanceOption::getSort));
        Verify.notNull(mtOptionMap);
        List<MtOptionTree> mtOptionTrees = new ArrayList<>();
        mtOptionMap.forEach((sort, options) -> {
            mtOptionTrees.add(new MtOptionTree(sort, options));
        });
        return RestResponse.success(mtOptionTrees);
    }

    /**
     * 维保项list
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("listOptions")
    @Verifier(fields = {"mtCompanyId", "type", "liftType"})
    public RestResponse listOptions(@Ver @RequestBody MtRecordRequest request) {
        Integer type = request.getType();
        Integer liftType = request.getLiftType();
        Long mtCompanyId = request.getMtCompanyId();
        List<MaintenanceOption> optionList = maintenanceOptionService.listByTypeAndLiftType(type, liftType, mtCompanyId);
        return RestResponse.success(optionList);
    }

    /**
     * 点击填写保养单，带收费项，新增record
     *
     * @param entity MaintenanceRecord
     * @return RestResponse
     */
    @PostMapping("tofill")
    public RestResponse tofill(@Valid @RequestBody MaintenanceRecord entity) {
        entity.setStatus("-2");
        entity.setIsRegular(0);
        //补录
        entity.setIsRepair(1);
        boolean result = maintenanceRecordService.tofill(entity);
        if (result) {
            return RestResponse.success(entity.getId());
        } else {
            return RestResponse.fail();
        }
    }

    /**
     * app端可对超期计划补录工单及评价，补录后需文员审核；
     *
     * @param entity MaintenanceRecord
     * @return RestResponse
     */
    @PostMapping("add")
    public RestResponse add(@Valid @RequestBody MaintenanceRecord entity) {
        /**
         * app端添加维保单
         * -3：没有维保记录（可以补录）null -> -3
         * -2：已补录工单，（没有维保项，可以继续填写）
         * -1：待审核（通过app端添加的），通过后台添加的，直接 ->3
         */
        if (StrUtil.isEmpty(entity.getMaintenanceOption())) {
            entity.setStatus("-2");
        } else {
            entity.setStatus("-1");
        }
        entity.setIsRegular(0);
        //补录
        entity.setIsRepair(1);
        boolean result = maintenanceRecordService.tofill(entity);
        if (result) {
            return RestResponse.success(entity.getId());
        } else {
            return RestResponse.fail();
        }
    }

    /**
     * 通过：app端提交的保养单点通过后生成正式保养单  status:-1 -> 3
     * 清除超期: 超期的计划修改status ->1,workDate -> now
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("pass")
    @Verifier(fields = {"id"})
    public RestResponse pass(@Ver @RequestBody MtRecordRequest request) {
        MaintenanceRecord entity = maintenanceRecordService.getById(request.getId());
        Verify.notNull(entity);
        Verify.notTrue("-1".equals(entity.getStatus()), "维保单必须待审核才能操作");
        entity.setStatus("3");
        boolean result = maintenanceRecordService.pass(entity);
        return RestResponse.success(result);
    }

    /**
     * 退回：app提交的保养单，点退回后，app端重新提交
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("refuse")
    @Verifier(fields = {"id"})
    public RestResponse refuse(@Ver @RequestBody MtRecordRequest request) {
        MaintenanceRecord entity = maintenanceRecordService.getById(request.getId());
        Verify.notNull(entity);
        Verify.notTrue("-1".equals(entity.getStatus()), "维保单必须待审核才能操作");
        boolean result = maintenanceRecordService.removeById(entity.getId());
        return RestResponse.success(result);
    }

    /**
     * 填写保养单-确认，带维保项和现场图片 ，修改record
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("fill")
    @Verifier(fields = {"id", "maintenanceOption", "imgs"})
    public RestResponse fill(@Ver @RequestBody MtRecordRequest request) {
        Long id = request.getId();
        MaintenanceRecord entity = maintenanceRecordService.getById(id);
        Verify.notNull(entity);

        String maintenanceOption = request.getMaintenanceOption();
        entity.setMaintenanceOption(maintenanceOption);
        entity.setStatus("0");
        boolean result = maintenanceRecordService.fill(entity, request.getImgs());
        return RestResponse.success(result);
    }

    /**
     * 补录工单，带收费项，修改record
     *
     * @param entity MaintenanceRecord
     * @return RestResponse
     */
    @PostMapping("workOrder")
    public RestResponse workOrder(@Valid @RequestBody MaintenanceRecord entity) {
        boolean result = maintenanceRecordService.tofill(entity);
        return RestResponse.success(result);
    }

    /**
     * 删除维保记录中的收费项
     *
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("deleteCost")
    @Verifier(fields = {"id", "mtRecordCostId"})
    public RestResponse deleteCost(@Ver @RequestBody MtRecordRequest request) {
        MtRecordCost mtRecordCost = mtRecordCostService.getById(request.getMtRecordCostId());
        if (Objects.isNull(mtRecordCost)) {
            return RestResponse.success(true);
        }
        MaintenanceRecord record = maintenanceRecordService.getById(request.getId());
        Verify.notNull(record);

        boolean result = maintenanceRecordService.deleteCost(record, mtRecordCost);

        return RestResponse.success(result);
    }

    /**
     * 维保评价，同时修改维保记录的status -> 3
     *
     * @param entity
     * @return
     */
    @PostMapping("evaluate")
    public RestResponse evaluate(@Valid @RequestBody Evaluation entity) {
        MaintenanceRecord record = maintenanceRecordService.getById(entity.getRecordId());
        Verify.notNull(record);
        //状态 1：已签到，未完成保养，2：完成保养，未选择评价方式，3：待评价，4：完成。
        Verify.notTrue("2".equals(record.getStatus()), "维保记录待评价才能操作");
        //来源（1 维保；2 急修）
        entity.setSource(1);
        entity.setLiftId(record.getLiftId());
        entity.setPostDate(LocalDateTime.now());

        //维保记录设置完成
        record.setStatus("3");
        boolean result = maintenanceRecordService.evaluate(record, entity);
        return RestResponse.success(result);
    }

    @PostMapping("pageByLift")
    @Verifier(fields = {"liftId", "projectId", "mtCompanyId"})
    public RestResponse pageByLift(@Ver @RequestBody MtRecordRequest request) {
        IPage<MtRecordResponse> pages = maintenanceRecordService.pageByLift(request);
        return RestResponse.success(pages);
    }
}
