package cn.com.ty.lift.push.app;

import cn.com.ty.lift.common.model.PushMessage;
import cn.com.ty.lift.common.utils.ValuePool;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.messaging.support.GenericMessage;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Component
@AllArgsConstructor
public class PushConsumer {

    private XingePush            xingePush;
    private JmsMessagingTemplate jmsMessagingTemplate;

    private class Processor {
        private String      destination;
        private PushMessage pushMessage;
        private boolean     result;

        private Processor(String destination, PushMessage pushMessage) {
            this.destination = destination;
            this.pushMessage = pushMessage;
        }

        private Processor perform() {
            String title = pushMessage.getTitle();
            String content = pushMessage.getContent();
            ArrayList<String> toList = pushMessage.getToList();
            switch (this.destination) {
                case ValuePool.QUEUE_ANDROID_TOKEN:
                    result = xingePush.pushTokenOnAndroid(title, content, toList);
                    break;
                case ValuePool.QUEUE_ANDROID_ALL:
                    result = xingePush.pushAllOnAndroid(title, content);
                    break;
                case ValuePool.QUEUE_IOS_TOKEN:
                    result = xingePush.pushTokenOnIos(title, content, toList);
                    break;
                case ValuePool.QUEUE_IOS_ALL:
                    result = xingePush.pushAllOnIos(title, content);
                    break;
                default:
                    result = true;
            }
            return this;
        }

        private void tryAgain() {
            int tryCount = pushMessage.getTryCount();
            if (!result && tryCount < ValuePool.PUSH_TRY_MAX_TIMES) {
                pushMessage.setTryCount(tryCount + 1);
                jmsMessagingTemplate.send(destination, new GenericMessage<>(pushMessage));
            }
        }
    }

    private void process(String destination, PushMessage pushMessage){
        if(pushMessage.needSplit()){
            List<ArrayList<String>> doSplit = pushMessage.doSplit(ValuePool.PUSH_MAX_SIZE);
            for (ArrayList<String> list : doSplit) {
                pushMessage.setToList(list);
                new Processor(destination, pushMessage).perform().tryAgain();
            }
        }else{
            new Processor(destination, pushMessage).perform().tryAgain();
        }
    }

    @JmsListener(destination = ValuePool.QUEUE_ANDROID_TOKEN)
    public void listenAndroidTokenQueue(PushMessage pushMessage) {
        log.info("listen {} receive message：{}",ValuePool.QUEUE_ANDROID_TOKEN, pushMessage);
        process(ValuePool.QUEUE_ANDROID_TOKEN, pushMessage);
    }

    @JmsListener(destination = ValuePool.QUEUE_ANDROID_ALL)
    public void listenAndroidAllQueue(PushMessage pushMessage) {
        log.info("listen {} receive message：{}",ValuePool.QUEUE_ANDROID_ALL, pushMessage);
        process(ValuePool.QUEUE_ANDROID_ALL, pushMessage);
    }

    @JmsListener(destination = ValuePool.QUEUE_IOS_TOKEN)
    public void listenIosTokenQueue(PushMessage pushMessage) {
        log.info("listen {} receive message：{}",ValuePool.QUEUE_IOS_TOKEN, pushMessage);
        process(ValuePool.QUEUE_IOS_TOKEN, pushMessage);
    }

    @JmsListener(destination = ValuePool.QUEUE_IOS_ALL)
    public void listenIosAllQueue(PushMessage pushMessage) {
        log.info("listen {} receive message：{}",ValuePool.QUEUE_IOS_ALL, pushMessage);
        process(ValuePool.QUEUE_IOS_ALL, pushMessage);
    }
}